package com.chinaitop.depot.basic.controller;

import java.text.SimpleDateFormat;
import java.util.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.model.BasicStorehouse;
import com.chinaitop.depot.basic.model.BasicWarehouse;
import com.chinaitop.depot.basic.model.BasicWarehouseExample;
import com.chinaitop.depot.basic.model.BasicWarehouseExample.Criteria;
import com.chinaitop.depot.basic.service.BasicStorehouseService;
import com.chinaitop.depot.basic.service.BasicWarehouseService;
import com.chinaitop.depot.utils.BusinessUtils;
import com.chinaitop.utils.ParameterUtil;
import com.chinaitop.utils.WarehouseDataSynchronization;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 货位管理Controller，货位管理菜单中所有的功能都在这里
 * @author User
 *
 */
@RestController
@RequestMapping(value="/Warehouse")
@Api(value= "BasicWarehouseController", description = "货位管理控制类")
public class BasicWarehouseController {

	final static Logger logger = LoggerFactory.getLogger(BasicWarehouse.class);

	@Resource
	private BasicWarehouseService basicWarehouseService;
	
	@Resource
	private BasicStorehouseService basicStorehouseService;
	/**
	 * 按条件查询 货位 信息
	 * 
	 * @param pageNum 页数
	 * @param pageSize 每页显示的条数
	 * @param storehouseId 仓房号
	 * @param warehouseName 货位名
	 * @param graindepotId 单位ID
	 * @param libraryType 粮库类型
	 * @return
	 */
	@RequestMapping(value="/getList", method = RequestMethod.GET)
	@ApiOperation(value="查询 货位 信息列表", notes = "查询 货位 信息列表，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "storehouseId", value = "仓房ID", paramType = "query"),
		@ApiImplicitParam(name = "warehouseId", value = "货位ID", paramType = "query"),
		@ApiImplicitParam(name = "warehouseName", value = "货位名称", paramType = "query"),
		@ApiImplicitParam(name = "graindepotId", value = "单位ID", paramType = "query"),
		@ApiImplicitParam(name = "libraryType", value = "粮库类型", paramType = "query"),
		@ApiImplicitParam(name = "delFlag", value = "是否删除", paramType = "query")
	})
	public PageInfo<BasicWarehouse> getList(Integer pageNum, Integer pageSize, Integer storehouseId, Integer warehouseId, 
			String warehouseName, Integer graindepotId, String libraryType, Integer delFlag) {
		BasicWarehouseExample wareExample = new BasicWarehouseExample();
		Criteria criteria = wareExample.createCriteria();
		List<BasicWarehouse> list = null;
		try {
			
			// 拼接查询条件
			if (!"".equals(warehouseName) && warehouseName != null) {
				criteria.andWarehouseNameLike("%"+warehouseName+"%");
			}
			if (null != storehouseId && !"".equals(storehouseId)) {
				criteria.andStorehouseIdEqualTo(storehouseId); //只查询顶级菜单
			}
			if (null != warehouseId && !"".equals(warehouseId) ){
				criteria.andWarehouseIdEqualTo(warehouseId);
			}
			if (null != libraryType) {
				//处理空仓报警问题
				if(libraryType.length() == 1){
					criteria.andLibraryTypeEqualTo(libraryType);
				}else if(libraryType.length() > 3){
					String[] libraryString = libraryType.substring(2).split(",");
					Integer[] libraryInteger = new Integer[libraryString.length];
					for (int i = 0; i < libraryString.length; i++) {
						libraryInteger[i] = Integer.parseInt(libraryString[i]);
					}
					List<Integer> result = Arrays.asList(libraryInteger);
					criteria.andWarehouseIdNotIn(result);
				}
			}

			/* 查询当前库下的数据 */
			criteria.andOrgIdEqualTo(graindepotId);

			if (null != pageNum && null != pageSize) {
				PageHelper.startPage(pageNum, pageSize);
			}
			
			// 值为1是没删除的数据(在用),0是已删除的数据(弃用)
			if (null != delFlag) {
				criteria.andDelFlagEqualTo(delFlag); 
			}
			wareExample.setOrderByClause("storehouse_id asc");
			list = basicWarehouseService.findCondition(wareExample);
		} catch (Exception e) {
			e.printStackTrace();
		}
		PageInfo<BasicWarehouse> pageInfo = new PageInfo<BasicWarehouse>(list);
		return pageInfo;
	}
	
	/**
	 * 按货位ID查询一条数据
	 * 
	 * @param id 数据ID
	 * @return
	 */
	@RequestMapping(value="/findByWarehouse", method = RequestMethod.GET)
	@ApiOperation(value="按货位ID查询一条数据", notes = "按货位ID查询一条数据")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "数据ID", paramType = "query")
	})
	public BasicWarehouse findByWarehouse(Integer id) {
		BasicWarehouse basicWarehouse = new BasicWarehouse();
		if (id != null) {
			basicWarehouse = basicWarehouseService.findByParmaryKey(id);
		}
		return basicWarehouse;
	}

	/**
	 * 更新货位信息
	 * 
	 * @param warehouseJson JSON数据对象
	 * @return
	 */
	@RequestMapping(value="/edit", method = RequestMethod.POST)
	@ApiOperation(value="更新货位信息", notes = "更新货位信息")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "warehouseJson", value = "JSON货位数据对象", paramType = "query"),
		@ApiImplicitParam(name = "userInfo", value = "JSON登录数据对象", paramType = "query")
	})
	public Map<String, Object> edit(HttpServletRequest request, String warehouseJson, String userInfo) {
		Map<String, Object> map = new HashMap<String, Object>();
		JSONObject user = null;
		Integer orgId = null;
        String userName = "";
        if (null != userInfo && !"".equals(userInfo)) {
        	user = JSONObject.parseObject(userInfo);
        	orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
        	userName = user.get("username") == null?"":user.get("username").toString();
        }

		Date date = new Date();
		// JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        try {
			BasicWarehouse warehouse = (BasicWarehouse)mapper.readValue(warehouseJson, BasicWarehouse.class);
			String hwdm = "";
			if (warehouse.getWarehouseId() != null) {
				warehouse.setUpdatename(userName);
				warehouse.setUpdatedate(date);
				BasicStorehouse storehouse = basicStorehouseService.findByPrimary(warehouse.getStorehouseId());
				warehouse.setHwdm(BusinessUtils.setBZhwbm(storehouse.getAjbm(), warehouse.getWarehouseCode()));
				basicWarehouseService.update(warehouse);
			} else {
				warehouse.setWarehouseStatus(2963); //添加时默认货位状态为空仓
				warehouse.setOrgId(orgId);
				if(!ParameterUtil.isequal(warehouse.getLibraryType(),"1")){
					warehouse.setDepotId(orgId);
				}
				if (!"".equals(warehouse.getWarehouseCode())) {
					BasicStorehouse storehouse = basicStorehouseService.findByPrimary(warehouse.getStorehouseId());
					hwdm = BusinessUtils.setBZhwbm(storehouse.getAjbm(), warehouse.getWarehouseCode());
					warehouse.setHwdm(hwdm.trim());
				}
				warehouse.setCreatename(userName);
				warehouse.setCreatedate(date);
				warehouse.setDelFlag(1);
				basicWarehouseService.add(warehouse);
			}
			/* 更新单位表中仓房数、油罐数、有效仓容、有效罐容 */
			basicStorehouseService.editOrgChNum(warehouse.getOrgId());
			map.put("status", "success");
		} catch (Exception e) {
			e.printStackTrace();
			map.put("status", "error");
            map.put("msg", "保存失败！");
		}
		
		return map;
	}

	/**
	 * 删除货位信息
	 * 
	 * @param id 数据对象
	 * @return
	 */
	@RequestMapping(value="/remove", method = RequestMethod.POST)
	@ApiOperation(value="删除货位信息", notes = "删除货位信息")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "数据ID", paramType = "form")
	})
	public Map<String, Object> remove(Integer id, Integer storehouseId) {
		Map<String, Object> map = new HashMap<String, Object>();
        try {
        	//先查询这条仓房数据,进行判断
        	BasicWarehouse basicWarehouse = basicWarehouseService.findByParmaryKey(id);
        	//先查询这条货位数据，通过判断如果为弃用状态则根据仓房状态进行修改,若仓房状态为应用状态,则不对货位状态进行修改
        	BasicStorehouse basicStorehouse = basicStorehouseService.findByPrimary(storehouseId);
        	Integer delFlag = basicWarehouse.getDelFlag();
        	if (delFlag == 1 || "1".equals(delFlag)) {
				delFlag = 0;
			} else if (delFlag == 0 || "0".equals(delFlag)) {
				delFlag = 1;
				//判断为弃用改应用状态,则进行应用
				basicStorehouse.setDelFlag(delFlag);
				basicStorehouseService.updateBasicStorehouse(basicStorehouse);
			}

        	basicWarehouse.setDelFlag(delFlag);
			if (id != null) {
				basicWarehouseService.update(basicWarehouse);;
			}
			/* 更新单位表中仓房数、油罐数、有效仓容、有效罐容 */
			basicStorehouseService.editOrgChNum(basicStorehouse.getOrgId());
			map.put("status", "success");
		} catch (Exception e) {
			e.printStackTrace();
			map.put("status", "error");
            map.put("msg", "操作失败！");
		}
		
		return map;
	}
	
	/**
	 * 根据单位和仓房查询货位信息（货位对外接口）
	 * 
	 * @param unitId 单位ID
	 * @param storehouseId 仓房ID
	 * @return
	 */
	@RequestMapping(value="/getWarehouse", method = RequestMethod.GET)
	@ApiOperation(value="根据单位和仓房查询货位信息（货位对外接口）", notes = "根据单位和仓房查询货位信息（货位对外接口）")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "unitId", value = "单位ID", paramType = "query"),
		@ApiImplicitParam(name = "storehouseId", value = "仓房ID", paramType = "query"),
		@ApiImplicitParam(name = "libraryType", value = "库类型(0主粮库,1代储库)", paramType = "query")
	})
	public Map<String, Object> getWarehouse(Integer unitId, Integer storehouseId, String libraryType) {
		Map<String, Object> map = null;
		
		map = basicWarehouseService.getWarehouse(unitId, storehouseId, libraryType);
		
		return map;
	}
	
	@RequestMapping(value="/findWareCode", method=RequestMethod.GET)
	@ApiOperation(value="防止货位编码重复添加", notes = "storehouseCode参数")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "storehouseId", value = "仓房ID", paramType = "query"),
			@ApiImplicitParam(name = "warehouseCode", value = "货位编码", paramType = "query"),
			@ApiImplicitParam(name = "wareId", value = "货位ID", paramType = "query"),
			@ApiImplicitParam(name = "libraryType", value = "判断是代储点还是主库", paramType = "query")
			})
	public String findWareCode(HttpServletRequest request, Integer storehouseId, String warehouseCode, Integer wareId, String libraryType) {
		String codeFlag = "";
		List<BasicWarehouse> list = null;
		Integer orgId = (Integer) request.getSession().getAttribute("orgId");
		BasicWarehouseExample warehouseExample = new BasicWarehouseExample();
		Criteria cria = warehouseExample.createCriteria();
		cria.andStorehouseIdEqualTo(storehouseId);
		cria.andWarehouseCodeEqualTo(warehouseCode);
		cria.andOrgIdEqualTo(orgId);
		cria.andWarehouseIdNotEqualTo(wareId);
		//cria.andLibraryTypeEqualTo(libraryType);
		try {
			list = basicWarehouseService.findCondition(warehouseExample);
			if (null != list && list.size() > 0) {
				codeFlag = "0";
			} else {
				codeFlag = "1";
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return codeFlag;
	}
	
	//库点信息同步
	@SuppressWarnings("all")
	@RequestMapping("/warehouseSynchronization")
	@ApiOperation(value="货位信息同步", notes = "货位信息同步")
	public void orgInfoSynchronization() {
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
		List list = WarehouseDataSynchronization.queryWarehouse();
		BasicWarehouseExample wareExample = new BasicWarehouseExample();
		List<BasicWarehouse> wareList = basicWarehouseService.findCondition(wareExample);
		for(BasicWarehouse warehouse : wareList) {
			int num = 0;
			for(int i=0;i<list.size();i++) {
				Map<String,Object> map = (Map<String,Object>)list.get(i);
				String hwbh = ObjectUtils.toString(map.get("hwbh"),"");//库点编码
				String refreshTime = ObjectUtils.toString(map.get("refreshTime"),"");
				if(hwbh.equals(ObjectUtils.toString(warehouse.getWarehouseCode(),""))) {
					if(!refreshTime.equals(format.format(warehouse.getUpdatedate()))) {
						WarehouseDataSynchronization.warehouseSynchronizationUpdate(map, warehouse);
					}
					break;
				}else if(num == (list.size()-1)){
					WarehouseDataSynchronization.warehouseSynchronizationInsert(warehouse);
				}else {
					num = num+1;
				}
			}
		}
	}

	@RequestMapping(value="/updateWarehouseCrkStatus", method=RequestMethod.POST)
	@ApiOperation(value="修改货位的出入库状态值", notes = "修改货位的出入库状态值")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "wareId", value = "货位ID", paramType = "form"),
		@ApiImplicitParam(name = "crk_status", value = "出入库状态(1空仓,2入库中,3封仓,4出库中,9其它)", paramType = "form")
	})
	public void updateWarehouseCrkStatus(Integer wareId, String crk_status) {

		String date = ParameterUtil.getSysDateTime();
		logger.info("时间："+date+"，封仓时更新出入库状态回写接口，封仓调用，接收参数是，货位ID："+wareId+"，状态值："+crk_status);
		try {
			if (null != wareId && null != crk_status && !"".equals(crk_status)) {
				BasicWarehouse warehouse = new BasicWarehouse();
				warehouse.setWarehouseId(wareId);
				warehouse.setCrkStatus(crk_status);
				
				//修改数据
				basicWarehouseService.update(warehouse);
				logger.info("时间："+date+"，封仓时更新出入库状态回写接口，封仓调用，调用回写成功！");
			} else {
				logger.info("时间："+date+"，封仓时更新出入库状态值crk_status出错，参数：货位ID="+wareId+"，出入库状态值="+crk_status);
			}
		} catch (NumberFormatException e) {
			logger.info("时间："+date+"，封仓时更新出入库状态回写接口，封仓调用，调用失败，报错如下！");
			e.printStackTrace();
		}
	}

	@SuppressWarnings("all")
	@RequestMapping(value="/getCountWhs", method=RequestMethod.GET)
	@ApiOperation(value="查询货位数", notes = "查询货位数")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgId", value = "单位ID", paramType = "query"),
		@ApiImplicitParam(name = "delFlag", value = "启用状态(1:启用,0:弃用)", paramType = "query")
	})
	public Integer getCountWhs(Integer orgId, Integer delFlag) {
		Integer hws = 0;
		try {
			hws = basicWarehouseService.getCountWhs(orgId, delFlag);
		} catch (Exception e) {
			hws = 0;
			e.printStackTrace();
		} finally {
			return hws;
		}
	}
}
