package com.chinaitop.depot.basic.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import com.chinaitop.depot.basic.mapper.BasicEnumMapper;
import com.chinaitop.depot.basic.model.*;
import com.chinaitop.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;

import org.apache.commons.lang.ObjectUtils;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.mapper.BasicStorehouseMapper;
import com.chinaitop.depot.basic.mapper.BasicTankMapper;
import com.chinaitop.depot.basic.mapper.BasicWarehouseMapper;
import com.chinaitop.depot.basic.model.BasicStorehouseExample.Criteria;
import com.chinaitop.depot.basic.service.BasicKeeperHouseService;
import com.chinaitop.depot.basic.service.BasicKeeperService;
import com.chinaitop.depot.basic.service.BasicStorehouseService;
import com.chinaitop.depot.basic.service.BasicWarehouseService;
import com.chinaitop.depot.utils.BusinessUtils;

@Service
@SuppressWarnings("all")
public class BasicStorehouseServiceImpl implements BasicStorehouseService {

	@Resource
	private BasicStorehouseMapper basic;

	@Resource
	private BasicWarehouseMapper wareMapper;

	@Resource
	private BasicWarehouseService warehouseService;

	@Resource
	private BasicEnumMapper enumMapper;

	@Resource
	private BasicKeeperHouseService keeperHouseService;

	@Resource
	private BasicKeeperService keeperService;
	
	@Resource
	private BasicTankMapper basicTankMapper;

	@Override
	public void addBasicStorehouse(BasicStorehouse basicStorehouse) throws Exception {
		basic.insertSelective(basicStorehouse);
	}

	@Override
	public void removeBasicStorehouse(Integer id) throws Exception {
		basic.deleteByPrimaryKey(id);
	}

	@Override
	public void updateBasicStorehouse(BasicStorehouse basicStorehouse) throws Exception {
		basic.updateByPrimaryKey(basicStorehouse);
	}

	@Override
	public BasicStorehouse findByPrimary(Integer id) throws Exception {
		return basic.selectByPrimaryKey(id);
	}

	@Override
	public List<BasicStorehouse> findBasicStorehouse(BasicStorehouseExample basicStorehouseExample) throws Exception {
		return basic.selectByExample(basicStorehouseExample);
	}

	@Override
	public List<BasicStorehouse> findBasicStorehouseNew(Map<String, Object> map) throws Exception {
		return basic.selectByExampleNew(map);
	}

	@Override
	public int count(BasicStorehouseExample basicStorehouseExample) throws Exception {
		return basic.countByExample(basicStorehouseExample);
	}

	@Override
	public Map<String, Object> getStorehouseData(Integer orgId, String libraryType) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		BasicStorehouseExample example = new BasicStorehouseExample();
		Criteria criteria = example.createCriteria();
		List<BasicStorehouse> houselist = null;
		try {
			example.setOrderByClause("storehouse_id asc");
			if (ParameterUtil.isnotnull(orgId)) { 
				criteria.andOrgIdEqualTo(orgId);
			}
			if (ParameterUtil.isnotnull(libraryType)) {
				criteria.andLibraryTypeEqualTo(libraryType);
			}else if(ParameterUtil.isnull(libraryType)){
				criteria.andLibraryTypeEqualTo("0");//如果不传值则默认查询主库数据，不查询代储库数据
			}
			criteria.andDelFlagEqualTo(1); // 查询正常数据
			houselist = basic.selectByExample(example); //获取当前单位下的所有有效数据
		} catch (Exception e) {
			e.printStackTrace();
		}
		modelMap.put("houseList", houselist);
		modelMap.put("houseObj", getStorehouseObj(orgId,libraryType));
		modelMap.put("houseCodeObj", getStorehouseCodeObj(orgId,libraryType));
		return modelMap;
	}

	/**
	 * 获取当前单位下所有仓房数据(全局接口，不可随便修改)
	 *
	 * @param orgId
	 * @return
	 */
	private Map<Integer, Object> getStorehouseObj(Integer orgId,String libraryType) {
		BasicStorehouseExample example = new BasicStorehouseExample();
		Criteria criteria = example.createCriteria();
		List<BasicStorehouse> houseObj = null;
		try {
			example.setOrderByClause("storehouse_id asc");

			if (ParameterUtil.isnotnull(libraryType)) {
				criteria.andLibraryTypeEqualTo(libraryType);
			}else if(ParameterUtil.isnull(libraryType)){
				criteria.andLibraryTypeEqualTo("0");//如果不传值则默认查询主库数据，不查询代储库数据
			}
			if (ParameterUtil.isnotnull(orgId)) {
				criteria.andOrgIdEqualTo(orgId);
			}
			
			/*criteria.andDelFlagEqualTo(1); // 查询正常数据
*/			houseObj = basic.selectByExample(example); //获取当前单位所有的仓房数据
		} catch (Exception e) {
			e.printStackTrace();
		}
		Map<Integer, Object> objMap = new HashMap<Integer, Object>();
		for (BasicStorehouse obj : houseObj) {
			objMap.put(obj.getStorehouseId(), obj);
		}
		return objMap;
	}

	/**
	 * 获取当前单位下所有仓房数据-仓房编码(全局接口，不可随便修改)
	 *
	 * @param depotId
	 * @return
	 */
	private Map<String, Object> getStorehouseCodeObj(Integer orgId,String libraryType) {
		BasicStorehouseExample example = new BasicStorehouseExample();
		Criteria criteria = example.createCriteria();
		List<BasicStorehouse> houseObj = null;
		try {
			example.setOrderByClause("storehouse_id asc");

			if (ParameterUtil.isnotnull(libraryType)) {
				criteria.andLibraryTypeEqualTo(libraryType);
			}else if(ParameterUtil.isnull(libraryType)){
				criteria.andLibraryTypeEqualTo("0");//如果不传值则默认查询主库数据，不查询代储库数据
			}
			if (ParameterUtil.isnotnull(orgId)) {
				criteria.andOrgIdEqualTo(orgId);
			}
			criteria.andDelFlagEqualTo(1); // 查询正常数据
			houseObj = basic.selectByExample(example); //获取当前单位所有的仓房数据
		} catch (Exception e) {
			e.printStackTrace();
		}
		Map<String, Object> objMap = new HashMap<>();
		for (BasicStorehouse obj : houseObj) {
			objMap.put(obj.getStorehouseCode(), obj);
		}
		return objMap;
	}

	@Override
	public String importData(List<List<String>> list, JSONObject user) {

		Integer orgId = null;
		String orgName = "";
		if (null != user) {
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			orgName = user.get("orgName") == null?"":user.get("orgName").toString();
		}

		String uuid = null;
		Date date = new Date();
		Map<String, Integer> map = new HashMap<String, Integer>();
		BasicStorehouse storehouse = null;
		BasicWarehouse warehouse = null;
		String msg = "";//消息
		boolean value_flag = false;
		int len = 0;//长度
		//根据字典名字查询字典编号
		BasicEnumExample enumExample = new BasicEnumExample();
		BasicEnumExample.Criteria Criteria = enumExample.createCriteria();
		List<BasicEnum> basicEnums;

		for (int i = 0; i < list.size(); i++) {
			List<String> revList = list.get(i);
			uuid = UUID.randomUUID().toString();

			for (int j = 0; j < revList.size(); j++) {
				switch(j) {
					case 0://仓房编号
						if (ParameterUtil.isnotnull(revList.get(j))) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 1 列的值的长度不能超过50！";
								return msg;
							}

							Integer mapKey = map.get(revList.get(j));
							if (mapKey == null) {
								storehouse = new BasicStorehouse();
								map.put(revList.get(j), null);
								storehouse.setStorehouseCode(revList.get(j));
							}
						}
						break;
					case 1://仓房名称
						if (storehouse != null) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 2 列的值的长度不能超过50！";
								return msg;
							}
							storehouse.setStorehouseName(revList.get(j));
						}
						break;
					case 2://仓房类型
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							enumExample.clear();
							Criteria = enumExample.createCriteria();
							Criteria.andEnumnameEqualTo(revList.get(j));
							basicEnums = enumMapper.selectByExample(enumExample);
							if(basicEnums.size() == 0){
								msg = "第 "+(i+1)+" 行第 3 列的值不正确，请查找后在导入！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 11) {
								msg = "第 "+(i+1)+" 行第 3 列的值的长度不能超过11！";
								return msg;
							}
							storehouse.setStorehouseType(basicEnums.get(0).getEnumid());
						}
						break;
					case 3://设计仓容(吨)
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							value_flag = revList.get(j).matches("[0-9]+");
							if (!value_flag) {
								msg = "第 "+(i+1)+" 行的第 4 列的值只能是数字！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 30) {
								msg = "第 "+(i+1)+" 行第 4 列的值的长度不能超过30！";
								return msg;
							}
							storehouse.setDesignCapacity(BigDecimal.valueOf(Integer.parseInt(revList.get(j))));
						}
						break;
					case 4://实际仓容(吨)
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							value_flag = revList.get(j).matches("[0-9]+");
							if (!value_flag) {
								msg = "第 "+(i+1)+" 行的第 5 列的值只能是数字！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 30) {
								msg = "第 "+(i+1)+" 行第 5 列的值的长度不能超过30！";
								return msg;
							}
							storehouse.setActualCapacity(BigDecimal.valueOf(Integer.parseInt(revList.get(j))));
						}
						break;
					case 5://通风方式
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							enumExample.clear();
							Criteria = enumExample.createCriteria();
							Criteria.andEnumnameEqualTo(revList.get(j));
							basicEnums = enumMapper.selectByExample(enumExample);
							if(basicEnums.size() == 0){
								msg = "第 "+(i+1)+" 行第 6 列的值不正确，请查找后在导入！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 100) {
								msg = "第 "+(i+1)+" 行第 6 列的值的长度不能超过100！";
								return msg;
							}
							storehouse.setDrafttype(basicEnums.get(0).getEnumid());
						}
						break;
					case 6://保管方式
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							enumExample.clear();
							Criteria = enumExample.createCriteria();
							Criteria.andEnumnameEqualTo(revList.get(j));
							basicEnums = enumMapper.selectByExample(enumExample);
							if(basicEnums.size() == 0){
								msg = "第 "+(i+1)+" 行第 7 列的值不正确，请查找后在导入！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 100) {
								msg = "第 "+(i+1)+" 行第 7 列的值的长度不能超过100！";
								return msg;
							}
							storehouse.setKeepingWay(basicEnums.get(0).getEnumid());
						}
						break;
					case 7://装粮线高度(m)
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							value_flag = revList.get(j).matches("[0-9]+");
							if (!value_flag) {
								msg = "第 "+(i+1)+" 行的第 8 列的值只能是数字！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 30) {
								msg = "第 "+(i+1)+" 行第 8 列的值的长度不能超过30！";
								return msg;
							}
							storehouse.setGrainHeigth(new BigDecimal(revList.get(j)));
						}
						break;
					/*case 8://保管员 注：liangs 2019/07/08 注掉保管员列
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 9 列的值的长度不能超过50！";
								return msg;
							}
							storehouse.setDutystoreman(revList.get(j));
						}
						break;*/
					case 8://仓房状态
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							enumExample.clear();
							Criteria = enumExample.createCriteria();
							Criteria.andEnumnameEqualTo(revList.get(j));
							basicEnums = enumMapper.selectByExample(enumExample);
							if(basicEnums.size() == 0){
								msg = "第 "+(i+1)+" 行第 9 列的值不正确，请查找后在导入！";
								return msg;
							}
							len = revList.get(j).length();
							if (len > 11) {
								msg = "第 "+(i+1)+" 行第 9 列的值的长度不能超过11！";
								return msg;
							}
							storehouse.setStorehouseState(basicEnums.get(0).getEnumid());
						}
						break;
					case 9://交付使用日期
						if (ParameterUtil.isnotnull(revList.get(j)) && storehouse != null) {
							if(!ParameterUtil.stringIsTime(revList.get(j))){
								msg = "第 "+(i+1)+" 行第 10 列的值不属于时间，请确认后再提交！";
								return msg;
							}
							storehouse.setUsedate(ParameterUtil.string2datetime(revList.get(j)));
						}
						if (storehouse != null) {
							storehouse.setUuid(uuid);
							storehouse.setDelFlag(1);
							storehouse.setCreatedate(date);
							storehouse.setOrgId(orgId);
						}
						break;
					case 10://货位编号
						if (ParameterUtil.isnotnull(revList.get(j))) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 11 列的值的长度不能超过50！";
								return msg;
							}

							warehouse = new BasicWarehouse();
							warehouse.setWarehouseCode(revList.get(j));
						}
						break;
					case 11://货位名称
						if (ParameterUtil.isnotnull(revList.get(j))) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 12 列的值的长度不能超过50！";
								return msg;
							}
							if (warehouse != null) {
								warehouse.setWarehouseName(revList.get(j));
							}
						}
						break;
					case 12://货位容量(吨)
						if (ParameterUtil.isnotnull(revList.get(j))) {
							len = revList.get(j).length();
							if (len > 50) {
								msg = "第 "+(i+1)+" 行第 13 列的值的长度不能超过50！";
								return msg;
							}
							if (warehouse != null) {
								warehouse.setWarehouseCapacity(new BigDecimal(revList.get(j)));
							}
						}
						if (warehouse != null) {
							warehouse.setDelFlag(1);
							warehouse.setCreatedate(date);
							warehouse.setOrgId(orgId);
							warehouse.setWarehouseStatus(2963); //添加时默认货位状态为空仓
						}
						break;
					case 13:
						if (ParameterUtil.isnotnull(revList.get(j))) {
							//插入仓房信息
							if (storehouse != null) {
								storehouse.setLibraryType(revList.get(j));
							}
							if (warehouse != null) {
								if("1".equals(revList.get(j))){
									warehouse.setLibraryType(revList.get(j));
								}
							}
						}
						break;
					case 14:
						//if (ParameterUtil.isnotnull(revList.get(j))) {
							Map<String,Object> depotMap = new HashMap<String, Object>();

							//插入仓房信息
							if (storehouse != null) {

								//判断是否有代储粮库
								if ("1".equals(storehouse.getLibraryType())) {
									depotMap.put("agentDepotName",revList.get(j));
									depotMap = basic.getAgentDepotId(depotMap);
									if(ParameterUtil.isnull(depotMap)){
										msg = "第 "+(i+1)+" 行第 15 列的值的不正确，请查询后再添加！";
										return msg;
									}
								}

								if(ParameterUtil.isequal("1",storehouse.getLibraryType())){
									storehouse.setDepotId(Integer.parseInt(depotMap.get("id").toString()));
									storehouse.setDepotName(revList.get(j));
									map.put(storehouse.getDepotName(), storehouse.getDepotId());
								}else{
									storehouse.setDepotId(orgId);
									storehouse.setDepotName(orgName);
								}
								basic.insertSelective(storehouse);
								map.put(storehouse.getStorehouseCode(), storehouse.getStorehouseId());
							}
							if (warehouse != null) {
								if(ParameterUtil.isequal("1",warehouse.getLibraryType())){
									warehouse.setDepotId(map.get(revList.get(j)));
									warehouse.setDepotName(revList.get(j));
								}else{
									warehouse.setDepotId(orgId);
									warehouse.setDepotName(orgName);
								}
								warehouse.setStorehouseId(map.get(revList.get(0)));
								wareMapper.insertSelective(warehouse);
							}
						//}
						storehouse = null;
						warehouse = null;
						break;
					default :
						break;

				}

			}

		}
		return msg;
	}

	@Override
	public String getStorehouseKeeper(Integer id, Integer orgId, boolean flag) {
		String keeperNames = "";

		/* 获取这个仓房的所有保管员的员工ID */
		List<Integer> keep_list = keeperHouseService.getKeeperId(orgId, id);

		//2、然后把这些人的翻译过来，展示出来
		if (null != keep_list && keep_list.size() > 0) {
			keeperNames = keeperService.getKeeperNames(keep_list);
			if (flag) {
				if (keeperNames.length() > 7) {
					keeperNames = keeperNames.substring(0, 7) + "...";
				}
			}
		}

		return keeperNames;
	}
	

	@Override
	public Map<String, Object> getThreeTempCheckList(Integer orgId) {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		Map<String, Object> map = new HashMap<String, Object>();
		List<BasicStorehouse> houselist = null;
		try {
			if (ParameterUtil.isnotnull(orgId)) {
				map.put("orgId", orgId);
			}
			map.put("libraryType", 0);//默认查询主库数据，不查询代储库数据
			map.put("delFlag", 1);//查询正常数据
			houselist = basic.getThreeTempCheckList(map); //获取当前单位下的所有有效数据
		} catch (Exception e) {
			e.printStackTrace();
		}
		modelMap.put("houseList", houselist);
		return modelMap;
	}

	@Override
	public Map<String, Object> saveOrUpdate(HttpServletRequest request, String storehouseJson, String warehouseJson, String userInfo) throws Exception {
		Map<String, Object> modelMap = new HashMap<String, Object>();
		JSONObject user = null;
		Integer orgId = null;
		String userName = "";
		if (null != userInfo && !"".equals(userInfo)) {
			user = JSONObject.parseObject(userInfo);
			orgId = Integer.parseInt(user.get("orgId") == null?"":user.get("orgId").toString());
			userName = user.get("username") == null?"":user.get("username").toString();
		}
		//获取单位信息
		String orgInfo = ObjectUtils.toString(request.getSession().getAttribute("orgInfo"),"");
		String dwbm = "";
		JSONObject org = null;
		if (null != orgInfo && !"".equals(orgInfo)) {
			org = JSONObject.parseObject(orgInfo);
			dwbm = org.get("creditCode") == null?"":org.get("creditCode").toString();
		}
		Date date = new Date();

		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		BasicStorehouse basicStorehouse = (BasicStorehouse) mapper.readValue(storehouseJson, BasicStorehouse.class);
		basicStorehouse.setDelFlag(1);

		String cfbm = "";//仓房编码
		String ajbm = "";//廒间编码

		if (!"".equals(basicStorehouse.getStorehouseCode())) {
			cfbm = BusinessUtils.setBZcfandygbm(dwbm, basicStorehouse.getStorehouseCode(), "0", orgId+"");
			ajbm = BusinessUtils.setBZajbm(cfbm, basicStorehouse.getStorehouseType()+"", "0");
			basicStorehouse.setCfbm(cfbm);
			basicStorehouse.setAjbm(ajbm);
		}
		if (basicStorehouse.getStorehouseId() == null) {
			String uuid = UUID.randomUUID().toString();
			//String uuid = UUID.randomUUID().toString().replaceAll("-", "");
			basicStorehouse.setUuid(uuid);
			
			if(!ParameterUtil.isequal(basicStorehouse.getLibraryType(),"1")){
				basicStorehouse.setDepotId(orgId);
			}
			basicStorehouse.setOrgId(orgId);
			basicStorehouse.setCreatename(userName);
			basicStorehouse.setCreatedate(date);
			basic.insertSelective(basicStorehouse);

		} else {
			if(!ParameterUtil.isequal(basicStorehouse.getLibraryType(),"1")){
				basicStorehouse.setDutystoreman(null);
			}
			basicStorehouse.setUpdatename(userName);
			basicStorehouse.setUpdatedate(date);
			basic.updateByPrimaryKey(basicStorehouse);
		}

		//更新货位数据
		warehouseService.saveOrUpdate(basicStorehouse, warehouseJson);

		//更新单位表仓房数、油罐数、有效仓容、有效罐容
		editOrgChNum(orgId);

		modelMap.put("status", "success");

		return modelMap;
	}

	/**
	 * 更新单位表中仓房数、油罐数、有效仓容、有效罐容
	 * @param orgId
	 */
	@Override
	public void editOrgChNum(Integer orgId) throws Exception {
		//查询当前单位仓房数
		BasicStorehouseExample cf_example = new BasicStorehouseExample();
		BasicStorehouseExample.Criteria cf_criteria = cf_example.createCriteria();
		cf_criteria.andOrgIdEqualTo(orgId);
		cf_criteria.andLibraryTypeEqualTo("0");
		cf_criteria.andDelFlagEqualTo(1);
		int ch_count = basic.countByExample(cf_example);
		
		//查询当前单位的油罐数
		BasicTankExample yg_example = new BasicTankExample();
		BasicTankExample.Criteria yg_criteria = yg_example.createCriteria();
		yg_criteria.andOrgIdEqualTo(orgId);
		//yg_criteria.andLibraryTypeEqualTo("0");
		yg_criteria.andDelFlagEqualTo(1);
		int yg_count = basicTankMapper.countByExample(yg_example);

		//查询当前单位仓房总实际容量和油罐总实际容量
		Map<String, Object> rl_map = basic.queryCrAndgw(orgId);
		

		//更新当前单位下仓房数、油罐数数、有效仓房、有效罐容
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("orgId", orgId);
		map.put("cf_count", ch_count);
		map.put("yg_count", yg_count);
		map.put("cr_count", rl_map.get("cr_count"));
		map.put("gr_count", rl_map.get("gr_count"));

		basic.editOrgCfAndHwNum(map);
	}

}
