package com.chinaitop.depot.basic.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

import com.chinaitop.depot.basic.mapper.BasicWarehouseMapper;
import com.chinaitop.depot.basic.model.BasicStorehouse;
import com.chinaitop.depot.basic.model.BasicWarehouse;
import com.chinaitop.depot.basic.model.BasicWarehouseExample;
import com.chinaitop.depot.basic.model.BasicWarehouseExample.Criteria;
import com.chinaitop.depot.basic.service.BasicWarehouseService;
import com.chinaitop.depot.utils.BusinessUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class BasicWarehouseServiceImpl implements BasicWarehouseService {

	@Resource
	private BasicWarehouseMapper mapper;
	
	@Override
	public void add(BasicWarehouse basicWarehouse) {
		mapper.insertSelective(basicWarehouse);
	}

	@Override
	public void update(BasicWarehouse basicWarehouse) {
		mapper.updateByPrimaryKeySelective(basicWarehouse);
	}

	@Override
	public void delete(Integer id) {
		mapper.deleteByPrimaryKey(id);
	}

	@Override
	public BasicWarehouse findByParmaryKey(Integer id) {
		return mapper.selectByPrimaryKey(id);
	}

	@Override
	public List<BasicWarehouse> findCondition(BasicWarehouseExample example) {
		return mapper.selectByExample(example);
	}

	@Override
	public Map<String, Object> getWarehouse(Integer unitId, Integer storehouseId, String libraryType) {
		Map<String, Object> map = new HashMap<String, Object>();
		BasicWarehouseExample example = new BasicWarehouseExample();
		Criteria criteria = example.createCriteria();
		List<BasicWarehouse> warelist = mapper.selectByExample(example);  //获取所有货位，与仓房和库点无关
		Map<Integer, Object> wareMap = new HashMap<Integer, Object>();
		if (null != warelist && warelist.size() != 0) {
			for (BasicWarehouse ware : warelist) {
				wareMap.put(ware.getWarehouseId(), ware);
			}
		} else {
			warelist = new ArrayList<BasicWarehouse>();
		}
		map.put("wares", wareMap); // 数据列表页面转换对象
		
		if (unitId != null && !"".equals(unitId)) {
			criteria.andOrgIdEqualTo(unitId);
		}
		if (storehouseId != null && !"".equals(storehouseId)) {
			criteria.andStorehouseIdEqualTo(storehouseId);
		}
		if (libraryType != null && !"".equals(libraryType)) {
			criteria.andLibraryTypeEqualTo(libraryType);
		}
		criteria.andDelFlagEqualTo(1);
		example.setOrderByClause("warehouse_id ASC");
		List<BasicWarehouse> list = mapper.selectByExample(example); //获取库点下的某个仓房中的的货位
		map.put("wareList", list); //货位下拉列表集合
		
		return map;
	}

	@Override
	public void deletEexample(Integer storehouseId) {
		//条件
		BasicWarehouseExample example = new BasicWarehouseExample();
		Criteria criteria = example.createCriteria();
		criteria.andStorehouseIdEqualTo(storehouseId);
		criteria.andDelFlagEqualTo(1);

		//要修改的值
		BasicWarehouse warehouse = new BasicWarehouse();
		warehouse.setDelFlag(0);
		mapper.updateByExampleSelective(warehouse, example);
	}

	@Override
	public Integer getCountWhs(Integer orgId, Integer delFlag) throws Exception {
		BasicWarehouseExample example = new BasicWarehouseExample();
		BasicWarehouseExample.Criteria criteria = example.createCriteria();
		criteria.andLibraryTypeEqualTo("0");//查询本库的，不查第三方库的
		if (null != orgId) {
			criteria.andOrgIdEqualTo(orgId);
		}
		if (null != delFlag) {
			criteria.andDelFlagEqualTo(delFlag);
		}
		Integer hws = mapper.countByExample(example);
		return hws;
	}

	@Override
	public void saveOrUpdate(BasicStorehouse storehouse, String warehouseJson) throws Exception {
		ObjectMapper mappers = new ObjectMapper();
		List<BasicWarehouse> list = mappers.readValue(warehouseJson, new TypeReference<List<BasicWarehouse>>(){});

		String hwdm = "";
		for (BasicWarehouse warehouse : list) {
			if (!"".equals(warehouse.getWarehouseCode())) {
				hwdm = BusinessUtils.setBZhwbm(storehouse.getAjbm(), warehouse.getWarehouseCode());
				warehouse.setHwdm(hwdm);
			}
			if (null == warehouse.getWarehouseId()) {//没有货位ID那就是新增
				warehouse.setStorehouseId(storehouse.getStorehouseId());
				warehouse.setCreatename(storehouse.getCreatename());
				warehouse.setCreatedate(storehouse.getCreatedate());
				warehouse.setOrgId(storehouse.getOrgId());
				mapper.insertSelective(warehouse);
			} else {
				warehouse.setUpdatename(storehouse.getUpdatename());
				warehouse.setUpdatedate(storehouse.getUpdatedate());
				mapper.updateByPrimaryKeySelective(warehouse);
			}
		}
	}

}
