package com.chinaitop.depot.unissoft.webservice;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.mapper.BasicWarehouseMapper;
import com.chinaitop.depot.basic.model.BasicWarehouse;
import com.chinaitop.utils.ParameterUtil;

@Service
public class BasicWarehouseWebServiceImpl implements BasicWarehouseWebService {

	final static Logger logger = LoggerFactory.getLogger(BasicWarehouse.class);

	@Resource
	private BasicWarehouseMapper warehouseMapper;

	@Override
	@SuppressWarnings("all")
	public String uploadCfStatus(String json) {
		Map<String, Object> result = new HashMap<String, Object>();

		String date = ParameterUtil.getSysDateTime();
		logger.info("时间："+date+"，出入库状态回写接口，出入库系统调用，接收参数是："+json);
		try {
			if (null != json) {
				/* 将获取到的json串转换成json数组 */
				JSONArray json_array = JSONArray.parseArray(json);
				JSONObject json_object = null;

				/* 遍历json数组，获取数组中值 */
				for (Iterator iterator = json_array.iterator(); iterator.hasNext();) {

					/* 获取json对象 */
					json_object = (JSONObject) iterator.next();
					if (json_object == null) {
						continue;
					}

					//仓房ID(云南没有油罐的业务，不用判断是否是油罐)
					String houseId = ObjectUtils.toString(json_object.get("chId"), "");
					//货位ID(云南没有油罐的业务，不用判断是否是油罐)
					String wareId = ObjectUtils.toString(json_object.get("hwId"), "");
					//如果是油罐，那么调过
					if (houseId.equals(wareId)) {
						continue;
					}
					//单位ID
					String orgId = ObjectUtils.toString(json_object.get("org_Id"), "");
					//货位状态值1空仓,2入库中,3封仓,4出库中,9其它
					String status = ObjectUtils.toString(json_object.get("status"), "");
					//业务时间
					String ywsj = ObjectUtils.toString(json_object.get("cjsj"), "");

					//修改数据
					Map<String, Object> param = new HashMap<String, Object>();
					param.put("wareId", Integer.parseInt(wareId));
					param.put("orgId", Integer.parseInt(orgId));
					param.put("status", status);
					param.put("ywsj", ywsj);
					
					/**
					 * 1、查询当前仓房货位有没有封仓；
					 * 2、如果封仓了，并且封仓时间在业务时间之前：那么就不回写当前传过来的状态，这个数据是在封仓前产生的
					 * 3、如果封仓了，并且封仓时间在业务时间之后：那么就回写当前传过来的状态，这个数据是在封仓后产生的
					 */
					//查询当前货位状态
					BasicWarehouse warehouse = warehouseMapper.selectByPrimaryKey(Integer.parseInt(wareId));
					if (null != warehouse && "3".equals(warehouse.getCrkStatus())) {
						//查询封仓时间
						List<Map<String, String>> map = warehouseMapper.queryFcdate(param);
						if (null != map && map.size() > 0) {
							String fcsj = map.get(0).get("filling_time")+" 00:00:00";
							//判断封仓时间在不在业务时间之前
							if (ParameterUtil.string2datetime(fcsj).after(ParameterUtil.string2datetime(ywsj))) {
								continue;
							} else {
								//回写状态
								warehouseMapper.updateWareStatus(param);
							}
						}
					} else {
						//回写状态
						warehouseMapper.updateWareStatus(param);
					}
				}
			}
			result.put("success", "true");
			result.put("msg", "上传成功");
			logger.info("时间："+date+"，出入库状态回写接口，出入库系统调用，调用成功！");
		} catch (NumberFormatException e) {
			e.printStackTrace();
			result.put("success", "false");
			result.put("msg", "上传失败");
			logger.info("时间："+date+"，出入库状态回写接口，出入库系统调用，调用失败！");
		}
		finally {
			JSONArray jsonArray = new JSONArray();
			jsonArray.add(result);
			return jsonArray.toString();
		}
	}

}
